<?php

namespace Drupal\fam_registration\Controller;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Utility\Token;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Route controller for generating registration welcome page content.
 */
class WelcomeController implements ContainerInjectionInterface {

  use MessengerTrait;

  /**
   * Configuration settings for the welcome page.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $welcomeSettings;

  /**
   * Creates a new instance of WelcomeController class.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config manager and factory.
   * @param \Drupal\Core\Utility\Token $token
   *   Token replacement service.
   * @param \Drupal\Component\Plugin\PluginManagerInterface $welcomeManager
   *   Plugin manager for registration welcome page plugins.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    protected Token $token,
    protected PluginManagerInterface $welcomeManager,
  ) {
    $this->welcomeSettings = $configFactory->get('fam_registration.welcome_page');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('token'),
      $container->get('plugin.manager.fam.welcome')
    );
  }

  /**
   * Get the page title to use for the registration welcome page.
   *
   * @return string
   *   Get string to used for the welcome page title.
   */
  public function getWelcomeTitle() {
    $title = $this->welcomeSettings->get('page_title') ?: 'Welcome to [site:name]';
    $title = Html::escape($title);

    return $this->token->replace($title);
  }

  /**
   * Create the page content for the registration welcome page.
   *
   * @return array
   *   Renderable array of the registration welcome page message.
   */
  public function buildWelcomePage() {
    if ($this->welcomeSettings->get('no_status')) {
      // Remove status messages from the user registration form. These messages
      // tend to be repetitive when welcome page content is displayed.
      $messenger = $this->messenger();
      $messenger->deleteByType(MessengerInterface::TYPE_STATUS);
    }

    $pluginId = $this->welcomeSettings->get('plugin') ?: 'message';
    $pluginConfig = $this->welcomeSettings->get('config') ?: [];
    $plugin = $this->welcomeManager->createInstance($pluginId, $pluginConfig);

    $build = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
          'registration-welcome',
          'registration-welcome--' . Html::cleanCssIdentifier($pluginId),
        ],
      ],

      'content' => $plugin->buildContent(),
    ];

    // Ensure that the welcome page has the correct caching information to
    // update when the welcome settings are updated.
    $metadata = BubbleableMetadata::createFromRenderArray($build);
    $metadata
      ->addCacheableDependency($this->welcomeSettings)
      ->addCacheableDependency($plugin)
      ->applyTo($build);

    return $build;
  }

}
