<?php

namespace Drupal\fam_registration\Form\Alter;

use Drupal\Core\Access\AccessManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Alter the standard login form based on FAM configurations.
 *
 * The FAM configurations can set options such as customized username and
 * password help text, in addition to helpful options such as registration
 * and forgot password links.
 */
class LoginFormAlter implements ContainerInjectionInterface, TrustedCallbackInterface {

  use StringTranslationTrait;

  /**
   * FAM login form settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $loginSettings;

  /**
   * Creates a new instance of the LoginFormAlter object.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The account instance for the currently logged in user.
   * @param \Drupal\Core\Access\AccessManagerInterface $accessManager
   *   The access manager service to check route access.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory and manager.
   */
  public function __construct(
    protected AccountInterface $account,
    protected AccessManagerInterface $accessManager,
    ConfigFactoryInterface $configFactory,
  ) {
    $this->loginSettings = $configFactory->get('fam_registration.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('access_manager'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks(): array {
    return [
      'addRegistrationLink',
    ];
  }

  /**
   * Alters the login form based on FAM settings.
   *
   * @param array $form
   *   Form structure and elements to alter.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state, build and values.
   */
  public function alterForm(array &$form, FormStateInterface $form_state): void {
    $account = $this->account;
    $accessCheck = $this->accessManager->checkNamedRoute(...);
    $hasRegistrationLink = !empty($this->loginSettings->get('login_form_placement'));

    // Adding the registration link in a pre-render callback.
    if ($account->isAnonymous() && $hasRegistrationLink && $accessCheck('user.register', [], $account)) {
      $form['#pre_render'][] = static::addRegistrationLink(...);
    }
  }

  /**
   * Element pre_render callback to place the login form registration link.
   *
   * @param array $element
   *   The form element structure to place the registration link to.
   *
   * @return array
   *   The form element with the registration link added and placed.
   */
  public function addRegistrationLink(array $element): array {
    $title = $this->loginSettings->get('login_form_register_text') ?: new TranslatableMarkup('Create an account');
    $registerLink = [
      '#type' => 'link',
      '#title' => $title,
      '#url' => Url::fromRoute('user.register'),
      '#attributes' => [
        'class' => ['register-link', 'button--secondary'],
      ],
    ];

    $placement = $this->loginSettings->get('login_form_placement');
    if ('actions' == $placement) {
      $element['actions']['register'] = $registerLink;
    }
    else {
      $element['register_link'] = [
        '#theme_wrappers' => ['container'],
        '#weight' => 'after' === $placement ? 110 : 20,
        '#attributes' => [
          'class' => ['fam-register-link'],
        ],
      ];

      $registerIntro = $this->loginSettings->get('login_form_register_intro');
      if (!empty($registerIntro)) {
        $element['register_link'][] = [
          '#plain_text' => $registerIntro,
          '#suffix' => ' ',
        ];
      }

      $element['register_link'][] = $registerLink;
    }

    return $element;
  }

}
