<?php

namespace Drupal\fam_registration\Form;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Menu\MenuLinkManagerInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure the new registration welcome page settings.
 */
class FamRegistrationSettingsForm extends ConfigFormBase {

  /**
   * The menu link manager.
   *
   * @var \Drupal\Core\Menu\MenuLinkManagerInterface
   */
  protected MenuLinkManagerInterface $menuLinkManager;

  /**
   * The registration welcome plugin manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected PluginManagerInterface $welcomeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    /** @var static $instance */
    $instance = parent::create($container)
      ->setMenuLinkManager($container->get('plugin.manager.menu.link'))
      ->setWelcomeManager($container->get('plugin.manager.fam.welcome'));

    return $instance;
  }

  /**
   * Sets the menu link manager service.
   *
   * @param \Drupal\Core\Menu\MenuLinkManagerInterface $menuLinkManager
   *   The menu link manager service.
   *
   * @return self
   *   The config form instance for method chaining.
   */
  public function setMenuLinkManager(MenuLinkManagerInterface $menuLinkManager): self {
    $this->menuLinkManager = $menuLinkManager;
    return $this;
  }

  /**
   * Set the registration welome plugin manager service.
   *
   * @param \Drupal\Component\Plugin\PluginManagerInterface $welcomeManager
   *   The registration welcome plugin manager.
   *
   * @return self
   *   The config form instance for method chaining.
   */
  public function setWelcomeManager(PluginManagerInterface $welcomeManager): self {
    $this->welcomeManager = $welcomeManager;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'fam_registration_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [
      'fam_registration.settings',
      'fam_registration.welcome_page',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);

    $settings = $this->config('fam_registration.settings');
    $welcomeConfig = $this->config('fam_registration.welcome_page');
    $pluginManager = $this->welcomeManager;

    // Determine the current values for plugin and the plugin config, form
    // could be rebuilding or this could be the first building of the form.
    if ($form_state->isRebuilding() && $form_state->getValue('plugin')) {
      $pluginId = $form_state->getValue('plugin');
      $pluginConfig = $form_state->getValue('config') ?: [];
    }
    else {
      $pluginId = $welcomeConfig->get('plugin') ?: 'message';
      $pluginConfig = $welcomeConfig->get('config') ?: [];
    }

    $loginInputSelector = 'select[name="registration[login_form_placement]"]';
    $registerIntro = $settings->get('login_form_register_intro');

    $form['registration'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Registration settings'),
      '#tree' => TRUE,
      '#description' => $this->t('Registration link can be added to the login form and user menu, but only if registration is enabled. See the <a href=":account_settings_url">user module settings</a> to configure registration for this site.', [
        ':account_settings_url' => Url::fromRoute('entity.user.admin_form')->toString(),
      ]),

      'login_form_placement' => [
        '#type' => 'select',
        '#title' => $this->t('Include registration link on login page'),
        '#empty_option' => $this->t('- No registration link -'),
        '#options' => [
          'actions' => $this->t('As a form action'),
          'before' => $this->t('After the login inputs'),
          'after' => $this->t('A link after the end of the form'),
        ],
        '#default_value' => $settings->get('login_form_placement'),
      ],
      'login_form_register_intro' => [
        '#type' => 'textfield',
        '#title' => $this->t('Content to appear before the registration link'),
        '#default_value' => $registerIntro ?? '',
        '#states' => [
          'visible' => [
            [$loginInputSelector => ['value' => 'before']],
            [$loginInputSelector => ['value' => 'after']],
          ],
        ],
      ],
      'login_form_register_text' => [
        '#type' => 'textfield',
        '#title' => $this->t('Login form registration link text'),
        '#placeholder' => $this->t('Create an account'),
        '#default_value' => $settings->get('login_form_register_text'),
        '#states' => [
          'invisible' => [
            'select[name="registration[login_form_placement]"]' => [
              'empty' => TRUE,
            ],
          ],
        ],
      ],
      'menu_register_enable' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Include registration link in the User menu'),
        '#default_value' => $settings->get('menu_register_enable'),
      ],
      'menu_register_text' => [
        '#type' => 'textfield',
        '#title' => $this->t('Register menu item title'),
        '#placeholder' => $this->t('Register'),
        '#default_value' => $settings->get('menu_register_text'),
        '#states' => [
          'visible' => [
            'input[name="registration[menu_register_enable]"]' => [
              'checked' => TRUE,
            ],
          ],
        ],
      ],
    ];

    // Fetch the possible welcome message plugins.
    $pluginOpts = [];
    foreach ($pluginManager->getDefinitions() as $def) {
      $pluginOpts[$def['id']] = $def['label'];
    }
    $pluginId = isset($pluginOpts[$pluginId]) ? $pluginId : key($pluginOpts);

    $form['welcome'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Welcome page'),
      '#tree' => TRUE,

      'enabled' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Enable registration welcome page'),
        '#default_value' => $welcomeConfig->get('enabled'),
        '#description' => $this->t('The registration page is only available when user registration is enabled by the <a href=":account_settings_url">user module settings</a>.', [
          ':account_settings_url' => Url::fromRoute('entity.user.admin_form')->toString(),
        ]),
      ],
      'no_status' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Hide the default status message (could be duplicative to welcome content).'),
        '#default_value' => $welcomeConfig->get('no_status'),
      ],
      'page_title' => [
        '#type' => 'textfield',
        '#title' => $this->t('Page title'),
        '#default_value' => $welcomeConfig->get('page_title') ?: NULL,
        '#size' => 128,
        '#maxlength' => 128,
      ],
      'plugin' => [
        '#type' => 'select',
        '#title' => $this->t('Welcome page type'),
        '#options' => $pluginOpts,
        '#access' => count($pluginOpts) > 1,
        '#default_value' => $pluginId,
      ],
      'config' => [
        '#parents' => ['welcome', 'config'],
        '#array_parents' => ['welcome', 'config'],
      ],
    ];

    if (count($pluginOpts) > 1) {
      $form['welcome']['plugin']['#ajax'] = [
        'wrapper' => 'registration-welcome-plugin-config',
        'callback' => static::class . '::updatePluginConfigAjax',
      ];
    }

    $subform = &$form['welcome']['config'];
    $plugin = $pluginManager->createInstance($pluginId, $pluginConfig);
    if ($plugin instanceof PluginFormInterface) {
      $subformState = SubformState::createForSubform($subform, $form, $form_state);
      $form['welcome']['config'] = $plugin->buildConfigurationForm($subform, $subformState);
    }
    else {
      $subform['#type'] = 'value';
      $subform['#value'] = [];
    }

    // Add a consistent wrapper around the configuration form elements,
    // this allows for AJAX wrapper targeting / replacing.
    $form['welcome']['config']['#prefix'] = '<div id="registration-welcome-plugin-config">';
    $form['welcome']['config']['#suffix'] = '</div>';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $pluginId = $form_state->getValue(['welcome', 'plugin']);
    $plugin = $this->welcomeManager->createInstance($pluginId, []);

    // If plugin generates a configuration form, give the plugin a chance to
    // validate all the configuration elements.
    if ($plugin instanceof PluginFormInterface) {
      $subForm = &$form['welcome']['config'];
      $subformState = SubformState::createForSubform($subForm, $form, $form_state);
      $plugin->validateConfigurationForm($subForm, $subformState);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $values = $form_state->getValues();

    // Save the link registration values.
    $settings = $this->config('fam_registration.settings');
    $originalSettings = $settings->getOriginal('', FALSE);
    $settings
      ->setData($values['registration'])
      ->save();

    // Build and save the welcome form page.
    $welcomeSettings = $values['welcome'];
    $welcomeConfig = $this->config('fam_registration.welcome_page');
    $plugin = $this->welcomeManager->createInstance($welcomeSettings['plugin'], []);

    // If plugin has a configuration form, allow it to transfer and process a
    // form values into the plugin configuration. This allows the plugin to
    // interpret and filter its own configuration form values.
    if ($plugin instanceof PluginFormInterface) {
      $subForm = &$form['welcome']['config'];
      $subformState = SubformState::createForSubform($subForm, $form, $form_state);
      $plugin->submitConfigurationForm($subForm, $subformState);
    }

    // Persist the configuration changes.
    $welcomeConfig->setData([
      'enabled' => $welcomeSettings['enabled'],
      'no_status' => $welcomeSettings['no_status'],
      'page_title' => $welcomeSettings['page_title'],
      'plugin' => $plugin->getPluginId(),
      'config' => $plugin->getConfiguration(),
    ])->save();

    // Needs to be checked after the new registration values have been saved.
    if ($values['registration'] != $originalSettings) {
      // If the registration link settings have changed, rebuild the menus so
      // the menu item changes can appear.
      $this->menuLinkManager->rebuild();
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * AJAX callback to update configurations when selected plugin is changed.
   *
   * @param array $form
   *   Reference to the built / rebuilt form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form build, input, stage and values.
   *
   * @return array|\Drupal\Core\Ajax\AjaxResponse
   *   Renderable array of content to replace the AJAX wrapper target.
   */
  public static function updatePluginConfigAjax(array &$form, FormStateInterface $form_state): array|AjaxResponse {
    return $form['welcome']['config'];
  }

}
