<?php

namespace Drupal\fam_registration\Plugin\FamWelcome;

use Drupal\Component\Plugin\Definition\PluginDefinitionInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\fam_registration\Attribute\FamWelcome;
use Drupal\fam_registration\Plugin\WelcomeBase;
use Drupal\filter\Render\FilteredMarkup;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Basic formatted text (WYSIWYG) and links for the registration welcome page.
 */
#[FamWelcome(
  id: 'message',
  label: new TranslatableMarkup('HTML Welcome Message'),
)]
class WelcomeMessage extends WelcomeBase implements PluginFormInterface, ContainerFactoryPluginInterface {

  /**
   * Creates an instance of RegistrationWelcomeMessage plugin class.
   *
   * @param array $configuration
   *   Plugin configuration, contains formatted text, filter format and links.
   * @param string $pluginId
   *   Machine name plugin ID.
   * @param array|\Drupal\Component\Plugin\Definition\PluginDefinitionInterface $pluginDefinition
   *   The plugin definition from plugin discovery.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory and manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   * @param \Drupal\Core\Utility\Token $token
   *   Token replacement manager.
   */
  public function __construct(
    array $configuration,
    string $pluginId,
    array|PluginDefinitionInterface $pluginDefinition,
    protected ConfigFactoryInterface $configFactory,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected LanguageManagerInterface $languageManager,
    protected Token $token,
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);

    // If empty configuration, apply the default message configurations.
    $this->configuration += $this->defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('language_manager'),
      $container->get('token')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    $userSettings = $this->configFactory->get('user.settings');
    $fallbackFormat = $this->configFactory
      ->get('filter.settings')
      ->get('fallback_format');

    $message = '';
    if ($userSettings->get('register') === UserInterface::REGISTER_VISITORS_ADMINISTRATIVE_APPROVAL) {
      $message = $this->t("Thank you for applying for an account.\nYour account is currently pending approval by the site administrator. You will receive a notification with instructions when your account is approved.");
    }
    elseif ($userSettings->get('verify_mail')) {
      $message = $this->t("Thank you for applying for an account.\nInstructions on how to reset your password and login have been sent to your email.");
    }
    else {
      // If neither admin approval or email verification is needed, users
      // should be logged in and never see our welcome page. This message is
      // likely to not be useful situation.
      $message = $this->t('Thank you for applying for an account!');
    }

    return [
      'message' => [
        'format' => $fallbackFormat,
        'value' => $message,
      ],
      'actions' => [
        'home' => [
          'enabled' => TRUE,
          'text' => $this->t('Back to homepage'),
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags(): array {
    $tags = parent::getCacheTags();
    $userSettings = $this->configFactory->get('user.settings');

    return Cache::mergeTags($tags, $userSettings->getCacheTags());
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration(): array {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration): void {
    $this->configuration = $configuration;
  }

  /**
   * Get definitions of links that can be placed on the welcome page.
   *
   * @return array
   *   Definitions of links that can be created for the registration welcome.
   *   Array key is the option array key, and contains the option label text,
   *   the button title, and the url.
   */
  protected function linkDefinitions(): array {
    return [
      'home' => [
        'label' => $this->t('Link to homepage'),
        'title' => $this->t('Back to homepage'),
        'url' => Url::fromRoute('<front>'),
      ],
    ];
  }

  /**
   * Build a renderable array of welcome page links or actions.
   *
   * @return array
   *   Renderable array of welcome page links or actions.
   */
  public function buildActions(): array {
    $enabledActions = $this->getConfiguration()['actions'] ?? [];
    $definitions = $this->linkDefinitions();

    $links = [];
    foreach ($enabledActions as $key => $action) {
      if (!empty($action['enabled']) && isset($definitions[$key])) {
        $links[$key] = [
          'title' => $action['title'] ?: $definitions[$key]['title'],
          'url' => $definitions[$key]['url'],
        ];
      }
    }

    return [
      '#theme' => 'links',
      '#links' => $links,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildContent(): array {
    $metadata = new BubbleableMetadata();
    $configuration = $this->getConfiguration();

    $filterFormatStorage = $this->entityTypeManager->getStorage('filter_format');
    $langCode = $this->languageManager
      ->getCurrentLanguage()
      ->getId();

    $message = $configuration['message']['value'];
    $formatId = $configuration['message']['format'];

    /** @var \Drupal\filter\FilterFormatInterface|null $format */
    $format = $filterFormatStorage->load($formatId);
    if (!$format || !$format->status()) {
      if ($format) {
        // If the original format exists, update content if this filter_format
        // is updated or enabled. This theoretically could mean we need to apply
        // this filter format instead of the fallback.
        $metadata->addCacheTags($format->getCacheTags());
      }

      // Use the fallback filter format if the requested format is missing
      // or is currently disabled.
      $fallbackFormat = $this->configFactory
        ->get('filter.settings')
        ->get('fallback_format');

      /** @var \Drupal\filter\FilterFormatInterface $format */
      $format = $filterFormatStorage->load($fallbackFormat);
    }

    $filters = $format->filters();

    // Give filters a chance to escape HTML-like data such as code or formulas.
    foreach ($filters as $filter) {
      if ($filter->status) {
        $message = $filter->prepare($message, $langCode);
      }
    }

    // Run the filter format filters on the content.
    foreach ($filters as $filter) {
      if ($filter->status) {
        $result = $filter->process($message, $langCode);
        $metadata = $metadata->merge($result);
        $message = $result->getProcessedText();
      }
    }

    $metadata->addCacheTags($format->getCacheTags());
    $metadata->addCacheContexts(['languages']);

    // Allow token replacements into the WYSIWYG content.
    $message = $this->token->replace($message, [], ['clear' => TRUE]);

    $html = [];
    $html['#markup'] = FilteredMarkup::create($message);
    $metadata->applyTo($html);

    return [
      '#theme' => 'fam_registration_welcome_message',
      '#body' => $html,
      '#actions' => $this->buildActions(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $configuration = $this->getConfiguration();

    $form['message'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Welcome message'),
      '#format' => $configuration['message']['format'],
      '#default_value' => $configuration['message']['value'],
    ];

    $form['actions'] = [
      '#type' => 'table',
      '#title' => $this->t('Action links'),
      '#header' => [
        '',
        $this->t('Action'),
        $this->t('Title'),
      ],
    ];

    foreach ($this->linkDefinitions() as $action => $def) {
      $form['actions'][$action] = [
        'enabled' => [
          '#type' => 'checkbox',
          '#title' => $this->t('Enable @action link', [
            '@action' => $def['label'],
          ]),
          '#title_display' => 'invisible',
          '#default_value' => $configuration['actions'][$action]['enabled'] ?? FALSE,
        ],
        'label' => [
          '#markup' => $def['label'],
        ],
        'title' => [
          '#type' => 'textfield',
          '#title' => $this->t('@action action text', [
            '@action' => $def['label'],
          ]),
          '#title_display' => 'invisible',
          '#default_value' => $configuration['actions'][$action]['title'] ?? $def['title'],
        ],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
    // No additional validation beyond the form element validation needed.
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $configuration['message'] = $form_state->getValue('message');
    $configuration['actions'] = array_filter($form_state->getValue('actions'));

    $this->setConfiguration($configuration);
  }

}
