<?php

namespace Drupal\fam\Form\Alter;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Exception\RouteNotFoundException;

/**
 * Helper class to alter the account edit form, with separate field edits forms.
 *
 * Class designed to edit the Drupal\user\AccountForm and generate operation
 * links for the fields configured to be edited on a separate page
 * (e.g. password and email).
 */
class AccountFormAlter implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * Configuration settings for the FAM for settings of the edit form.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $settings;

  /**
   * Create a new instance of the AccountFormAlter class.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Routing\RouteProviderInterface $routeProvider
   *   The route provider.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity type field manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend to store the discover password validation users fields.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    protected RouteProviderInterface $routeProvider,
    protected EntityFieldManagerInterface $entityFieldManager,
    protected CacheBackendInterface $cache,
  ) {
    $this->settings = $configFactory->get('fam.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('router.route_provider'),
      $container->get('entity_field.manager'),
      $container->get('cache.discovery')
    );
  }

  /**
   * Get the user field names that require current password validation.
   *
   * Get a list of field identifiers of field which have the
   * Drupal\user\Plugin\Validation\Constraint\ProtectedUserFieldConstraint
   * set on them. These are fields that need current password validation, and
   * can determine if the current password field is still needed on the edit
   * account form.
   *
   * @return array<string,string>
   *   Get an array of field names of user entity fields that have the
   *   ProtectedUserFieldConstraint set on them. Both the key and value of the
   *   array are the field name identifier.
   *
   * @see Drupal\user\Plugin\Validation\Constraint\ProtectedUserFieldConstraint
   */
  protected function getPasswordValidatedFields(): array {
    $cid = 'fam:password_validate_fields';

    if ($cached = $this->cache->get($cid)) {
      $protectedUserFields = $cached->data;
    }
    else {
      $userFields = $this->entityFieldManager->getBaseFieldDefinitions('user');
      $userFields += $this->entityFieldManager->getFieldDefinitions('user', 'user');

      $protectedUserFields = [];
      foreach ($userFields as $fieldName => $fieldDef) {
        foreach ($fieldDef->getConstraints() as $constraint => $constraintDef) {
          if ($constraint === 'ProtectedUserField') {
            $protectedUserFields[$fieldName] = $fieldName;
            break;
          }
        }
      }

      $this->cache->set($cid, $protectedUserFields, CacheBackendInterface::CACHE_PERMANENT, [
        'entity_field_info',
        'entity_types',
      ]);
    }

    return $protectedUserFields;
  }

  /**
   * Alter the account edit form based on the "fam.settings".
   *
   * @param array $form
   *   Reference to the form elements and structure to alter.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param \Drupal\user\UserInterface $account
   *   User entity of the account being edited by the form.
   */
  public function alterForm(array &$form, FormStateInterface $form_state, UserInterface $account): void {
    // Move the account name to the top of the form, if not explicitly set.
    $elements = &$form['account'];
    $elements['name']['#weight'] = $elements['name']['#weight'] ?? -1;

    try {
      $needsCurrentPass = $this->getPasswordValidatedFields();

      if ($this->settings->get('email_form')) {
        $this->routeProvider->getRouteByName('fam.user.edit_mail');
        $editEmailUrl = Url::fromRoute('fam.user.edit_mail', [
          'user' => $account->id(),
        ]);

        $elements['change_email_link'] = [
          '#theme' => 'fam_operation_link',
          '#op_name' => 'edit_email',
          '#title' => $this->t('Email'),
          '#text' => $this->t('Change Email'),
          '#url' => $editEmailUrl,
          '#value' => $account->getEmail(),
          '#weight' => $elements['mail']['#weight'] ?? 0,
        ];

        $elements['mail']['#access'] = FALSE;
        unset($needsCurrentPass['mail']);
      }

      if ($this->settings->get('password_form')) {
        $this->routeProvider->getRouteByName('fam.user.edit_pass');
        $editPassUrl = Url::fromRoute('fam.user.edit_pass', [
          'user' => $account->id(),
        ]);

        $elements['pass_reset_link'] = [
          '#theme' => 'fam_operation_link',
          '#op_name' => 'edit_pass',
          '#title' => $this->t('Password'),
          '#text' => $this->t('Change Password'),
          '#url' => $editPassUrl,
          '#field_access' => $account->get('pass')->access('edit'),
          '#weight' => $elements['pass']['#weight'] ?? 0,
        ];

        $elements['pass']['#access'] = FALSE;
        unset($needsCurrentPass['pass']);
      }

      // If we don't have any user fields left which need current password
      // validation, hide the "current_pass" form element.
      if (!$needsCurrentPass && !empty($elements['current_pass'])) {
        $elements['current_pass']['#access'] = FALSE;
      }
    }
    catch (RouteNotFoundException $e) {
      // Route subscriber failed to add these routes, keep original edit fields.
    }
  }

}
