<?php

namespace Drupal\fam\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Entity\EntityConstraintViolationListInterface;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\AccountForm;
use Drupal\user\Plugin\Validation\Constraint\ProtectedUserFieldConstraint;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Base from for editing specific user entity fields.
 *
 * This base form allows the creation of user account forms meant for editing
 * user fields without needing to build the complete AccountForm edit. This
 * avoids EntityFormDisplay loading, rendering and validation.
 */
abstract class EditUserFieldForm extends AccountForm {

  /**
   * Construct a new instance of the EditUserFieldForm class.
   *
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   Account of the administrator editing this account's roles. This will
   *   usually be the current user.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepo
   *   The entity repository service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entityTypeBundleInfo
   *   The entity type bundle info service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The date-time time service.
   */
  public function __construct(
    protected AccountInterface $currentUser,
    ModuleHandlerInterface $moduleHandler,
    EntityTypeManagerInterface $entityTypeManager,
    EntityRepositoryInterface $entityRepo,
    EntityTypeBundleInfoInterface $entityTypeBundleInfo,
    LanguageManagerInterface $languageManager,
    TimeInterface $time,
  ) {
    parent::__construct($entityRepo, $languageManager, $entityTypeBundleInfo, $time);

    $this->entityTypeManager = $entityTypeManager;
    $this->moduleHandler = $moduleHandler;
    $this->operation = static::getFormOperation();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('current_user'),
      $container->get('module_handler'),
      $container->get('entity_type.manager'),
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('language_manager'),
      $container->get('datetime.time')
    );
  }

  /**
   * Fetch a machine name string for the user edit operation of this form.
   *
   * @return string
   *   Get a machine name identifier for the edit operation of this string. This
   *   ID is used as part of the form_id and the user entity operation returned
   *   by ContentEntityForm::getOperation().
   */
  abstract protected static function getFormOperation(): string;

  /**
   * List the user fields being edited and needing constraint validation.
   *
   * Fields listed here are the form_state values copied to the entity during
   * entity building / rebuilding and validation flaggging.
   *
   * @return string[]
   *   A list of field names of field edited by this form.
   */
  abstract protected static function getEditFields(): array;

  /**
   * Create the user edit field elements for this user edit operation form.
   *
   * @param array $form
   *   Reference to the complete form elements and structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state, build and values.
   *
   * @return array
   *   FAPI elements for editing the user field values.
   */
  abstract protected function buildFieldElements(array &$form, FormStateInterface $form_state): array;

  /**
   * {@inheritdoc}
   */
  protected function getEditedFieldNames(FormStateInterface $form_state): array {
    return static::getEditFields();
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseFormId(): ?string {
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'user_form_' . static::getFormOperation();
  }

  /**
   * {@inheritdoc}
   */
  public function getOperation(): string {
    // Force the operation of the entity to the customized field edit operation.
    return static::getFormOperation();
  }

  /**
   * {@inheritdoc}
   */
  public function setOperation($operation): self {
    // Does nothing, operation is hard set for EditUserFieldForm subclasses.
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function init(FormStateInterface $form_state): void {
    EntityForm::init($form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function copyFormValuesToEntity(EntityInterface $entity, array $form, FormStateInterface $form_state): void {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $editFields = static::getEditFields();

    // Then extract the values of fields by simply copying from top-level form
    // values. This leaves the fields that are not being edited untouched.
    foreach ($editFields as $name) {
      $values = $form_state->getValue($name);

      if ($entity->hasField($name)) {
        $entity->set($name, $values);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, ?UserInterface $user = NULL): array {
    if (!$this->getEntity()) {
      if ($user) {
        $this->setEntity($user);
      }
      else {
        throw new NotFoundHttpException();
      }
    }

    // Use EntityForm instead of the ContentEntityForm to avoid building the
    // User EntityFormDisplay form widgets from being created.
    return EntityForm::buildForm($form, $form_state);
  }

  /**
   * Indicates that this form operations requires current password validation.
   *
   * Some user fields require current password validation in order to save
   * changes. This is to protect secure fields from being changed without proper
   * validation that the current user is authorized to make these changes.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state, build, storage and values.
   * @param bool $is_required
   *   Should the current password field be required.
   *
   * @return array
   *   Form element for entering current password when needed for account
   *   validation when altering fields that require the user password to update.
   *
   * @see \Drupal\user\Plugin\Validation\Constraint\ProtectedUserFieldConstraint
   */
  protected function buildCurrentPass(FormStateInterface $form_state, $is_required = TRUE): array {
    $account = $this->getEntity();
    $form_state->set('user', $account);

    return [
      '#type' => 'password',
      '#title' => $this->t('Current password'),
      '#size' => 25,
      '#weight' => -5,
      '#required' => $is_required,
      '#access' => !$form_state->get('user_pass_reset'),
      '#attributes' => ['autocomplete' => 'off'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {
    // Add the custom form elements for the user fields subclassed forms handle.
    $form['account'] = $this->buildFieldElements($form, $form_state);

    // Content entity forms do not use the parent's #after_build callback
    // because they only need to rebuild the entity in the validation and the
    // submit handler because Field API uses its own #after_build callback for
    // its widgets.
    //
    // @see \Drupal\Core\Entity\ContentEntityForm::form()
    $form = EntityForm::form($form, $form_state);
    unset($form['#after_build']);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state): array {
    $account = $this->getEntity();

    $actions['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#submit' => [
        '::submitForm',
        '::save',
      ],
    ];

    $actions['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => $account->toUrl('edit-form'),
      '#attributes' => [
        'class' => ['button', 'button--secondary'],
      ],
    ];

    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function buildEntity(array $form, FormStateInterface $form_state): EntityInterface {
    /** @var \Drupal\user\UserInterface $account */
    $account = ContentEntityForm::buildEntity($form, $form_state);

    // Set existing password if set in the form state if current password
    // validation is needed for this form.
    if (isset($form['account']['current_pass'])) {
      $current_pass = trim($form_state->getValue('current_pass'));

      if (strlen($current_pass) > 0) {
        $account->setExistingPassword($current_pass);
      }

      // Skip the protected user field constraint if the user came from the
      // password recovery page.
      // @phpstan-ignore-next-line variable.undefined
      $account->_skipProtectedUserFieldConstraint = $form_state->get('user_pass_reset');
    }

    return $account;
  }

  /**
   * {@inheritdoc}
   */
  protected function flagViolations(EntityConstraintViolationListInterface $violations, array $form, FormStateInterface $form_state): void {
    $editFields = static::getEditFields();

    // Flag entity level violations for the field edited by this form.
    foreach ($violations->getByFields($editFields) as $violation) {
      $propertyPath = ($violation->getConstraint() instanceof ProtectedUserFieldConstraint)
        ? 'current_pass' : str_replace('.', '][', $violation->getPropertyPath());

      // Allow violation messages to render HTML formatted violation text.
      $form_state->setErrorByName($propertyPath, $violation->getMessage());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    /** @var \Drupal\user\UserInterface $account */
    $account = $this->getEntity();
    $messenger = $this->messenger();

    if ($this->hasAccountChanged($account)) {
      $retVal = parent::save($form, $form_state);
      $messenger->addStatus($this->t('Account has been successfully updated.'));
    }
    else {
      $messenger->addWarning($this->t('No account changes were requested. No updates were applied.'));
      $retVal = 0;
    }

    $form_state->setRedirectUrl($account->toUrl('edit-form'));

    return $retVal;
  }

  /**
   * Check if the user account fields were updated.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user entity being updated by this form.
   *
   * @return bool
   *   TRUE if this user entity has any updated fields, FALSE otherwise.
   */
  protected function hasAccountChanged(UserInterface $user): bool {
    if (!$user->isNew()) {
      /** @var \Drupal\user\UserInterface $original */
      $original = $this->entityTypeManager
        ->getStorage('user')
        ->loadUnchanged($user->id());

      foreach (static::getEditFields() as $field) {
        try {
          if ($original->get($field)->getValue() != $user->get($field)->getValue()) {
            return TRUE;
          }
        }
        catch (\InvalidArgumentException $e) {
        }
      }
      return FALSE;
    }

    return TRUE;
  }

}
