<?php

namespace Drupal\fam\Form;

use Drupal\Component\Plugin\Discovery\CachedDiscoveryInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Menu\LocalTaskManagerInterface;
use Drupal\Core\Routing\RouteBuilderInterface;
use Drupal\fam\UserFieldHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form for editing the FAM settings config.
 */
class FamSettingsForm extends ConfigFormBase {

  /**
   * The route builder.
   *
   * @var \Drupal\Core\Routing\RouteBuilderInterface
   */
  protected RouteBuilderInterface $routeBuilder;

  /**
   * The local task plugin manager.
   *
   * @var \Drupal\Core\Menu\LocalTaskManagerInterface
   */
  protected LocalTaskManagerInterface $localTaskManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    /** @var static $instance */
    $instance = parent::create($container)
      ->setRouteBuilder($container->get('router.builder'))
      ->setLocalTaskManager($container->get('plugin.manager.menu.local_task'));

    return $instance;
  }

  /**
   * Set the route builder services for the config settings form.
   *
   * @param \Drupal\Core\Routing\RouteBuilderInterface $routeBuilder
   *   The route builder service to use for resetting route cache.
   *
   * @return self
   *   Return the form instance for method chaining.
   */
  public function setRouteBuilder(RouteBuilderInterface $routeBuilder): self {
    $this->routeBuilder = $routeBuilder;
    return $this;
  }

  /**
   * Set the local task manager instance.
   *
   * @param \Drupal\Core\Menu\LocalTaskManagerInterface $localTasKManager
   *   The local task manager service to reset the local task cache.
   *
   * @return self
   *   Return the form instance for method chaining.
   */
  public function setLocalTaskManager(LocalTaskManagerInterface $localTasKManager): self {
    $this->localTaskManager = $localTasKManager;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'edit_fam_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [
      'fam.settings',
    ];
  }

  /**
   * Get local task information about tabs that can have their title altered.
   *
   * @return array
   *   Editable account local tasks, keyed by local task ID, and contains the
   *   default title form Drupal, and a form label for the edit textfield.
   */
  protected function getEditableAccountTasks(): array {
    return [
      'entity.user.canonical' => [
        'label' => $this->t('View account'),
        'default' => $this->t('View'),
      ],
      'entity.user.edit_form' => [
        'label' => $this->t('Edit account'),
        'default' => $this->t('Edit'),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);
    $famSettings = $this->config('fam.settings');

    $form['form_fields'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Profile edit settings'),
      '#tree' => FALSE,

      'password_form' => [
        '#type' => 'select',
        '#title' => $this->t('Edit account <strong>password</strong> on'),
        '#options' => UserFieldHelper::getFieldEditOptions(),
        '#default_value' => $famSettings->get('password_form'),
      ],
      'email_form' => [
        '#type' => 'select',
        '#title' => $this->t('Edit account <strong>email</strong> on'),
        '#options' => UserFieldHelper::getFieldEditOptions(),
        '#default_value' => $famSettings->get('email_form'),
      ],
      'admin_theme' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Use admin theme?'),
        '#default_value' => $famSettings->get('admin_theme') ?? TRUE,
      ],
    ];

    $form['login_form'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Login form settings'),
      '#tree' => TRUE,

      'username_link' => [
        '#type' => 'value',
        '#value' => FALSE,
      ],
      'password_link' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Include a link to password recovery in the password field description?'),
        '#default_value' => $famSettings->get('login_form')['password_link'] ?? FALSE,
      ],
    ];

    // Create table for changing local task (tabs) for account operations.
    $table = [
      '#type' => 'table',
      '#header' => [
        $this->t('Tab'),
        $this->t('Label'),
      ],
    ];

    foreach ($this->getEditableAccountTasks() as $taskId => $task) {
      $table[$taskId] = [
        'label' => [
          '#markup' => $task['label'],
        ],
        'title' => [
          '#type' => 'textfield',
          '#placeholder' => $task['default'],
        ],
      ];
    }

    $taskLabels = $famSettings->get('local_tasks') ?: [];
    foreach ($taskLabels as $task) {
      if (isset($table[$task['id']]['title'])) {
        $table[$task['id']]['title']['#default_value'] = $task['title'];
      }
    }

    $form['tasks_wrapper'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Account tab titles'),
      '#description' => $this->t('Change the titles of the local tasks (tabs) which appear on user view and edit pages. Leaving a field blank will use the default Drupal titles.'),
    ];
    $form['tasks_wrapper']['local_tasks'] = $table;

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $famSettings = $this->config('fam.settings');
    $isRouteChanged = FALSE;

    // Check if any of the separate account field edit forms are enabled.
    foreach (['password_form', 'email_form'] as $opt) {
      $value = \intval($form_state->getValue($opt), 10);

      if ($famSettings->get($opt) != $value) {
        $famSettings->set($opt, $value);
        $isRouteChanged = TRUE;
      }
    }

    // Record the login form settings.
    $famSettings->set('admin_theme', $form_state->getValue('admin_theme'));
    $famSettings->set('login_form', $form_state->getValue('login_form'));

    // Filter out only local tasks where a label is provided by the admin.
    // All other local tasks should just use the defaults from Drupal core.
    $tasks = [];
    foreach ($form_state->getValue('local_tasks') as $taskId => $data) {
      if (!empty($data['title'])) {
        $tasks[] = [
          'id' => $taskId,
          'title' => $data['title'],
        ];
      }
    }
    $famSettings->set('local_tasks', $tasks);
    $famSettings->save();

    // If the separate edit password or edit email forms enabled options have
    // changed, update the route, local tasks and expire password routes.
    if ($isRouteChanged) {
      $this->routeBuilder->setRebuildNeeded();
      Cache::invalidateTags(['user_policies.password_expire_route']);
    }
    if ($this->localTaskManager instanceof CachedDiscoveryInterface) {
      $this->localTaskManager->clearCachedDefinitions();
    }

    $this->messenger()->addStatus($this->t('Configuration changes have been updated.'));
  }

}
