<?php

namespace Drupal\fam\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\fam\Event\FamEvents;
use Drupal\fam\Event\PassResetActionsEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * The dedicated password reset form.
 *
 * Form is used for password resets and expirations when the
 * "fam.settings::password_form" config option is set as enabled.
 */
class PassResetForm extends EditUserFieldForm {

  /**
   * The event dispatcher service.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected EventDispatcherInterface $eventDispatcher;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    /** @var static */
    return parent::create($container)->setEventDispatcher($container->get('event_dispatcher'));
  }

  /**
   * Set the event dispatcher service.
   *
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $eventDispatcher
   *   The event dispatcher service to set for this form instance.
   *
   * @return self
   *   The form instance to use for method chaining.
   */
  public function setEventDispatcher(EventDispatcherInterface $eventDispatcher): self {
    $this->eventDispatcher = $eventDispatcher;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks(): array {
    return array_merge(parent::trustedCallbacks(), ['alterPasswordConfirmTitles']);
  }

  /**
   * {@inheritdoc}
   */
  protected static function getFormOperation(): string {
    return 'edit_password';
  }

  /**
   * {@inheritdoc}
   */
  protected static function getEditFields(): array {
    return ['pass'];
  }

  /**
   * {@inheritdoc}
   */
  protected function buildFieldElements(array &$form, FormStateInterface $form_state): array {
    $elements = [];
    $account = $this->getEntity();

    if ($account->id() == $this->currentUser->id()) {
      $request = $this->getRequest();
      $elements['current_pass'] = $this->buildCurrentPass($form_state);

      // To skip the current password field, the user must have logged in via a
      // one-time link and have the token in the URL. Store this in $form_state
      // so it persists even on subsequent Ajax requests.
      if (!$form_state->get('user_pass_reset') && ($token = $request->get('pass-reset-token'))) {
        $sessToken = $request->getSession()->get('pass_reset_' . $account->id());
        $form_state->set('user_pass_reset', $sessToken && hash_equals($sessToken, $token));
      }

      if ($form_state->get('user_pass_reset')) {
        $elements['current_pass']['#access'] = FALSE;
      }
      else {
        $elements['current_pass']['#description'] = $this->t('Your current password is required in order for you to change your password. @link.', [
          '@link' => Link::fromTextAndUrl($this->t('Reset your password'), Url::fromRoute('user.pass'))->toString(),
        ]);
      }
    }

    $elements['pass'] = [
      '#type' => 'password_confirm',
      '#size' => 25,
      '#required' => TRUE,
      '#description' => $this->t('To change the current user password, enter the new password in both fields.'),
      '#pre_render' => [static::class . '::alterPasswordConfirmTitles'],
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state): array {
    $actions = parent::actions($form, $form_state);

    /** @var \Drupal\user\UserInterface $account */
    $account = $this->getEntity();
    $event = new PassResetActionsEvent($actions, $form, $form_state, $account);
    $this->eventDispatcher->dispatch($event, FamEvents::PASS_RESET_ACTIONS);

    return $actions;
  }

  /**
   * Password confirm element pre_render callback to update textfield titles.
   *
   * @param array $element
   *   Password confirm element to update the textfield titles for.
   *
   * @return array
   *   The altered render element for the password confirm element.
   */
  public static function alterPasswordConfirmTitles(array $element): array {
    $element['pass1']['#title'] = t('New password');
    $element['pass2']['#title'] = t('Confirm new password');

    return $element;
  }

}
