<?php

namespace Drupal\fam;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormErrorHandler;
use Drupal\Core\Form\FormErrorHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Form Error Handler decorator, to format login errors more clearly.
 *
 * Error handler to specifically alter the behavior of user login errors. Meant
 * to be used as a service decorator to keep whatever the configured behavior
 * or error handler service intact for all other forms and errors.
 *
 * For the login error specifically, users are often confused when only the
 * username is flagged as having an error, when the problem could be the
 * username or the password.
 */
class LoginFormErrorHandler extends FormErrorHandler implements FormErrorHandlerInterface {

  use MessengerTrait;
  use StringTranslationTrait;

  /**
   * Create a new instance of the LoginFormErrorHandler class.
   *
   * @param \Drupal\Core\Form\FormErrorHandlerInterface $errorHandler
   *   The original form_error_handler currently set. This could either be the
   *   core Drupal form error handler or the inline form error handler.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(
    protected FormErrorHandlerInterface $errorHandler,
    protected ConfigFactoryInterface $configFactory,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function handleFormErrors(array &$form, FormStateInterface $form_state): self {
    $formObject = $form_state->getFormObject();
    $formId = $formObject->getFormId();

    // Special handling for the user login form to display errors from
    // authentication errors (incorrect username or password). Falls back to
    // just using the default form error handler to display errors.
    ($formId === 'user_login_form' && !$form_state->get('uid'))
      ? $this->handleLoginAuthErrors($form, $form_state)
      : $this->errorHandler->handleFormErrors($form, $form_state);

    return $this;
  }

  /**
   * Handling for UserLoginForm to mark user name and password errors clearer.
   *
   * @param array $form
   *   Reference to current form elements and structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The complete form state with the form errors.
   *
   * @return self
   *   Returns itself for method chaining.
   */
  protected function handleLoginAuthErrors(array &$form, FormStateInterface $form_state): self {
    $errors = $form_state->getErrors();

    // Drupal core places flood or authentication errors on the username
    // form element. If login markes this element has having error, place the
    // error into the top of the login form, and flag the appropriate elements.
    if (isset($errors['name'])) {
      $nameErrors = $errors['name'];
      $form['name']['#error_no_message'] = TRUE;
      unset($errors['name']);

      if ($errors) {
        $form_state->clearErrors();

        foreach ($errors as $key => $error) {
          $form_state->setErrorByName($key, $error);
        }

        $this->errorHandler->handleFormErrors($form, $form_state);
        $form_state->setErrorByName('name', $nameErrors);
        $form['name']['#errors'] = $nameErrors;
      }
      else {
        $this->setElementErrorsFromFormState($form, $form_state);
      }

      // The Drupal\user\Form\UserLoginForm::finalValidation() will report a
      // check for the flood trigger setting to determine whether or not to
      // display general, username or password mismatch, flag password as well.
      if (!$form_state->get('flood_control_triggered')) {
        $form['pass']['#error_no_message'] = TRUE;
        $form['pass']['#errors'] = $nameErrors;
      }

      // Place the login authentication errors at the top of the form instead
      // of in the normal status message block.
      $form['login_errors'] = [
        '#theme' => 'status_messages',
        '#message_list' => [
          'error' => [$form['name']['#errors']],
        ],
        '#weight' => -99,
      ];
    }
    else {
      // Some other login related error, use the default error handler.
      $this->errorHandler->handleFormErrors($form, $form_state);
    }

    return $this;
  }

}
