<?php

namespace Drupal\fam\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Exception\RouteNotFoundException;

/**
 * Create the local task for the password and email forms when enabled.
 *
 * Based on the settings from the "fam.settings" config create the local tasks
 * for the enabled separate account operation forms.
 */
class AccountOpTaskDeriver extends DeriverBase implements ContainerDeriverInterface {

  use StringTranslationTrait;

  /**
   * Create a new local task object deriver.
   *
   * @param string $basePluginId
   *   The plugin ID of the deriver definition.
   * @param \Drupal\Core\Routing\RouteProviderInterface $routeProvider
   *   The route provider.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory and manager.
   */
  public function __construct(
    protected string $basePluginId,
    protected RouteProviderInterface $routeProvider,
    protected ConfigFactoryInterface $configFactory,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id): static {
    return new static(
      $base_plugin_id,
      $container->get('router.route_provider'),
      $container->get('config.factory')
    );
  }

  /**
   * Get info about local tasks this deriver creates for FAM account operations.
   *
   * @return array
   *   Array information about local tasks keyed by the route_name of the route
   *   these local tasks point to. Value contains a "setting_name" and "title"
   *   information that can be used to create the derived local task.
   */
  protected function getOpRoutes(): array {
    return [
      'fam.user.edit_pass' => [
        'setting_name' => 'password_form',
        'title' => $this->t('Change Password'),
      ],
      'fam.user.edit_mail' => [
        'setting_name' => 'email_form',
        'title' => $this->t('Update Email'),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition): array {
    $settings = $this->configFactory->get('fam.settings');
    $base = $base_plugin_definition;
    unset($base['deriver']);

    foreach ($this->getOpRoutes() as $routeName => $info) {
      if ($settings->get($info['setting_name'])) {
        try {
          $this->routeProvider->getRouteByName($routeName);
          $this->derivatives[$routeName] = [
            'title' => $info['title'],
            'route_name' => $routeName,
          ] + $base;
        }
        catch (RouteNotFoundException $e) {
          // Route doesn't exist, skip creating this tab.
        }
      }
    }

    return parent::getDerivativeDefinitions($base_plugin_definition);
  }

}
