<?php

namespace Drupal\fam\Routing;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\fam\Controller\PasswordResetController;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Route subscriber to optionally add password and email edit routes.
 *
 * FAM module allows optional password and email edit forms that need to be
 * dynamically added to the the route collection when enabled.
 *
 * Enabling the password form option also requires a change to the
 * "user.reset.login" route to redirect to the FAM provided password reset, as
 * password edits will be hidden on the regular account edit form.
 */
class UserEditRouteSubscriber extends RouteSubscriberBase {

  /**
   * FAM configuration settings for the route options.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $settings;

  /**
   * Creates a new instance of the UserEditRouteSubscriber instance.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory and fetcher.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->settings = $configFactory->get('fam.settings');
  }

  /**
   * {@inheritdoc}
   */
  protected function alterRoutes(RouteCollection $collection): void {
    $routeReqs = [
      'user' => '\d+',
      '_entity_access' => 'user.update',
    ];

    $options = [];
    $options['_admin_route'] = (bool) ($this->settings->get('admin_theme') ?? TRUE);
    $options['parameters'] = [
      'user' => ['type' => 'entity:user'],
    ];

    // Separate password form has been enabled.
    if ($this->settings->get('password_form')) {
      $defaults = [
        '_title' => 'Change Password',
        '_form' => '\Drupal\fam\Form\PassResetForm',
        'entity_type_id' => 'user',
      ];

      $pwRoute = new Route('/user/{user}/edit/password', $defaults, $routeReqs, $options);
      $collection->add('fam.user.edit_pass', $pwRoute);

      // Alter the route associated with the one time login page in order to
      // redirect the user to FAM password reset form when it is enabled.
      if ($route = $collection->get('user.reset.login')) {
        $route->setDefault('_controller', PasswordResetController::class . '::resetPassLogin');
      }
    }

    // Separate email form has been enabled.
    if ($this->settings->get('email_form')) {
      $defaults = [
        '_title' => 'Update Email',
        '_form' => '\Drupal\fam\Form\EditEmailForm',
        'entity_type_id' => 'user',
      ];

      $emailRoute = new Route('/user/{user}/edit/email', $defaults, $routeReqs, $options);
      $collection->add('fam.user.edit_mail', $emailRoute);
    }
  }

}
